%macro CPSTART( p1, mode=menu,  guiver=NEW,
                    pdb=,
                    remprof=,
                    share=,
                    access=,
                    disp=,
                    root=,
                    rootserv=,
                    system=,
                    sitelib=,
                    siteshr=,
                    siteacc=,
                    prompt=no,
                    internal=,
                    convert=NO,
                    verrep=NO,
                    mxglib=,
                    mxgsrc=,
                    _rc=);
 * If you add a parm here, update CPRIMCMD below;
 %*+----------------------------------------------------------------+
   | Copyright (c) 1997 by SAS Institute Inc., Cary, NC 27512-8000
   | Name:     CPSTART.SAS
   | Support:  JTR
   | Product:  IT Service Vision
   | Language: SAS
   | Purpose:  Setup IT Service Vision environment
   |
   | Parameters:
   |
   |   Name   Value              Description
   |
   |  mode=    MENU       Invoke IT Service Vision menu system GUI (prompt if
   |                        PDB not found)
   |  mode=    BATCH      Initialize IT Service Vision for batch (fail if PDB
   |                        not found)
   |
   |  guiver=  NEW        Invoke IT Service Vision old menu system if NEW, old
   |                      system if OLD
   |
   |  pdb=     [location] Initial PDB to be activated, default PDB
   |                        is the last PDB activated.
   |
   |  remprof= [name]    Remote server profile (SHARE/CONNECT)
   |
   |  share=   [server]   SAS/SHARE server for PDB
   |
   |  access=  WRITE      Write access mode (Unix) for PDB
   |  access=  READONLY   Readonly access mode (Unix) for PDB
   |
   |  disp=    OLD        Old (write) Disposition mode (MVS) for PDB
   |  disp=    SHR        Share (read) Disposition mode (MVS) for PDB
   |
   |  root=    [location] Library containing IT Service Vision software.
   |
   |  rootserv=[server]   SAS/SHARE server for root libraries (default
   |                        is READONLY with no server).
   |
   |  system=  [name]     Operating system (OSYS, MVS)
   |
   |  sitelib= [location] SITELIB location
   |
   |  siteacc= [mode]     Access mode for SITELIB
   |
   |  siteshr= [server]   SAS/SHARE server for SITELIB
   |
   |  prompt=  NO         Do not do version conversion
   |  prompt=  YES        Do version conversion if required
   |  prompt=  REPAIR     internal fix up setting
   |
   |  verrep=  NO         Do not do version report
   |
   |  internal=           Various internal options
   |  convert= NO         Sitelib override
   |
   |  mxglib=  [location] Name of MXG format library.
   |
   |  mxgsrc=  [location] Name of MXG source library.
   |
   |  _rc=     [name]     Name of macro variable to contain RC
   |
   | Output:   see purpose
   | Usage:
   | History:    date    action
   |             10Jul92 Creation from CSSTART.                             DMF
   |             03Mar94 D0040111 Rewrite CPSTART in SCL (INTERNAL=EXP).    DMF
   |             21Oct94 D0040111 Changed CPSTART parm (INTERNAL=EXP)       DAF
   |                     to (INTERNAL=OLD) to make old CPSTART work         DAF
   |             31Oct94 D0040111 Removed (intrenal=OLD) and old macro code DAF
   |                              Added CPVER macro suport                  DAF
   |             07Nov94 D0040111 Added CONVERT= parameter.                 DAF
   |             08Nov94 D0040111 Added VERREP= parameter.                  DAF
   |             02Mar95 D0040111 Added SITEACC= SITESHR= and changed DM... DAF
   |             03Mar95 D0040111 Made _rc parameter obsolete               DAF
   |             13Mar95 D0040111 Change cpsystem assignment for quick st   TXD
   |             23Mar95 D0040111 Set CPSYSTEM from %CPHSTTYP if it is      DMF
   |                     blank (which means CPE was not started previously).DMF
   |             02Jun95 D0077982 code before EXIT to eliminate             DAF
   |                     call execute messages.                             DAF
   |             25Aug95 D0086920 Upcase SYSTEM parm and remove stop stmts  CDW
   |             04Dec95 D0090244 Add list of valid commands for CPE cmd.   DMF
   |             02Feb96 D0092548 Compress blanks out of MXGSRC so that     SXM
   |                     it will not be truncated in SCL                    SXM
   |             20feb96 D0089490 Made _rc parameter NOT obsolete           JTR
   |             04mar96 D0095883 Added CPSOW global macro                  JTR
   |             14mar96 D0096377 Cleaned up parameter descriptions         JTR
   |             16May96 D0099364 Globalled the _rc user variable           CDW
   |             24Feb97 D0119452 Add support for Windows to CPOPSYS list.  DMF
   |             02Apr97 D0122691 Add remote profile support for Unix/Win   JTR
   |             09May97 D0054796 Add log scan to catch return codes we do  DMF
   |                     not catch otherwise.                               DMF
   |             09May97 D0116392 Change SAS/CPE to ITSV in all.            DMF
   |             13Jun97 D0126962 Default to GUIVER=OLD for MVS             JTR
   |             21Aug97 D0133177 Add to global macro list                  JTR
   | D0137220    29Sep97 Add CPUNBUF to global macro list                   CES
   | D0138374    08Oct97 Assert OPTIONS NOFMTERR                            CES
   |
   | Notes:
   | end
   +----------------------------------------------------------------+;

   options compress=yes obs=max firstobs=1 nofmterr;

   %************************************************************************;
   %* Macro variable declarations                                           ;
   %************************************************************************;

   %global cpver;            %* Version of portable IT Service Vision ;
   %global cplevel;          %* Level of host system CPE product ;
   %global cpe_up;           %* Y means IT Service Vision is up;
   %global cprimcmd;         %* List of valid parms for CPE command;
   %global cpconvrt;         %* Convert parameter YES | NO | REPAIR ;
   %global cpverrep;         %* Version report YES | NO ;
   %global cpsystem;         %* System running on;
   %global cprstat;          %* Design Rpt completion return code;
   %global cpnobs;           %* Flag to test for number of observations;
   %global cpprefix;         %* Prefix based on IT Service Vision product;
   %global cppdb;            %* Performance data base directory name;
   %global cproot;           %* IT Service Vision software directory;
   %global cpsite ;          %* Sitelib path name;
   %global cprootsh;         %* Server name for cproot libraries;
   %global cpsitesh;         %* Server name for site libraries;
   %global cpsiteac;         %* Site access option;
   %global cp_nmsg;          %* 2=Show all NOTEs,1=Show some, 0=show none;
   %if %str(&cp_nmsg)=%str() %then %let cp_nmsg=1;
   %global cppdbshr;         %* PDB share option;
   %global cppdbacc;         %* PDB access option;
   %global cppdbdsp;         %* PDB disp option;
   %global csdebug;          %* 1=Show debug info;
   %global csdelcol;         %* 2=del both col+col_ ds, 1=del col_ ds only;
   %global cpcolls;          %* List of all supported collectors;
   %global cpmxglib;         %* MXG format library;
   %global cpmxgsrc;         %* MXG format library;
   %global cpopsys;          %* Where CPE is running OSYS, WIN, or MVS;
   %global cpmode;           %* Menu, batch, or nomenu from mode= parm;
   %global cpguiver;         %* NEW or OLD version of GUI;
   %global cpjobid;          %* User info;
   %global cpusrnm;          %* User info;
   %global cpusrlc1;         %* User info;
   %global cpusrlc2;         %* User info;
   %global cpusrphn;         %* User info;
   %global cpsow;            %* Start-of-week setting;
   %global cplrmax;          %* Max RC value from ITSV macros;
   %global profname;         %* Remote server profile;
   %global cpunbuf;          %* UNBUFFERED option in table views;


   %* Macro vars used in reporting;
   %global cpbegin  cpend    cpredlvl cprptnm  cpugseg  cpulog
           cpuout   cpupgms  cpusrc   cpwhere  cpyval   cp_wmsg;
   %let cpyval=16;

   %local upusropt;          %* Update of user options needed;
   %let upusropt=N;
   %local uppdbopt;          %* Update of PDB options needed;
   %let uppdbopt=N;
   %local tabinfo;           %* Y=table template exists, N=does not;
   %local varinfo;           %* Y=table template exists, N=does not;
   %local vredinfo;          %* Y=table template exists, N=does not;
   %local tredinfo;          %* Y=table template exists, N=does not;
   %local i;
   %local resetdic resetopt; %* Flags to reset parms and dict;
   %local cpsserv;           %* Server for cpsyslib or access=readonly;
   %local pgmserv;           %* Server for pgmlib or access=readonly;
   %local pdbacc pdbshr;     %* PDB share and access options;
   %local pdbview rootview;  %* PDB and root SHARE options for views;
   %local usrinfo;           %* Y=copy this SAS ds from PGMLIB to SITELIB;
   %local mailinfo;          %* Y=copy this SAS ds from PGMLIB to SITELIB;
   %local cphday;            %* Y=copy this SAS ds from PGMLIB to SITELIB;
   %local message;           %* Y=copy this SAS ds from PGMLIB to SITELIB;
   %local tskinfo;           %* Y=copy this SAS ds from PGMLIB to SITELIB;
   %local cpsam;             %* Y=copy this SAS catlg from PGMLIB to SITELIB;
   %local siteshr;           %* Access mode for SITELIB;
   %local sitedflt;          %* Name of default sitelib location;
   %local siteacc;           %* Access mode of sitelib;
   %local library;           %* How each host refers to a SAS libarary;
   %local scanrc;            %* Used to hold log scan RC;

   %************************************************************************;
   %* Tables of product and host operating system info                      ;
   %* This also appears in CPSTART.SCL program (make changes to both places);
   %************************************************************************;
   %local opsysts roots products prefixes;
   %let opsysts  = %STR(OSYS                 WIN            MVS);
   %let roots    = %STR(!sasroot/saspgm/cpe/ !sasroot\cpe\  WORK.CPE.) ;
   %let products = %STR(OSYS                 WIN            MVS);
   %let prefixes = %STR(CS                   CW             CM);


   %************************************************************************;
   %* Establish list of valid parms for CPE command (lives in SASHELP.CPE.  ;
   %* PRIMARY.PROGRAM).  The program uses the macro value to id valid cmds. ;
   %* Trailing _ on command means CPE will upcase the value.  The value     ;
   %* must be < 200 chars or you will have to rewrite parts of cpe.program. ;
   %************************************************************************;
   %let cprimcmd = MODE_ PDB REMPROF_ ROOT SYSTEM_ SHARE ACCESS_ DISP_ SITESHR_
 SITEACC_ PROMPT_ INTERNAL_ SITELIB ROOTSERV GUIVER_
 CONVERT_ VERREP_ MXGLIB MXGSRC RC;

   %*************************************************************;
   %* Parse internal parm before anything else                   ;
   %*************************************************************;
   %let internal=%QUPCASE(&internal);
   %if %index(%STR( &internal ),%STR( NOP ))
      %then %goto EXIT;

   %put NOTE: CPE2001 IT Service Vision initialization in progress.;
   %put NOTE: CPE2001 CPRED004 Reduction fix-pack installed.;
   %global cppmmod;
   %global cppmguiv;
   %global cppmpdb;
   %global cppmroo;
   %global cppmsys;
   %global cppmshr;
   %global cppmacc;
   %global cppmdsp;
   %global cppmpmt;
   %global cppmint;
   %global cppmsit;
   %global cppmrts;
   %global cppmmxl;
   %global cppmmxs;
   %global cppmrc;

   %*********************************************************;
   %* Establish internal return code                         ;
   %*********************************************************;
   %if %length(&_rc)=0 %then %do;
      %local rc;
      %let _rc=rc;
   %end;
   %else %do;
      %global &_rc;
      %if %upcase(&mode)^=BATCH %then %do;
         %put WARNING:  Returned _RC value should not be used except;
         %put           when MODE=BATCH.;
         %end;
      %end;
   %let &_rc=0;


   ***************************************************************;
   * Setup scan of SAS log to set retcode from it.                ;
   ***************************************************************;
   %let scanrc=.;
   %CPLRSTAR ( _rc=scanrc );
   %if &scanrc>=8 %then %do;
      %put ERROR: Unable to route SAS log for error check.  SCANRC=&scanrc..;
      %let &_rc=&scanrc;
      %goto COMP_RC;
   %end;


   %*************************************************************;
   %* Establish access to CPE root                               ;
   %*************************************************************;
   %if %STR(&rootserv)^=%STR()
       %then %do;
          %let cprootsh=%STR(server=&rootserv);
          %let rootview=%STR(rmtview=NO);
       %end;
       %else %do;
          %let cprootsh=%STR();
          %let rootview=%STR();
       %end;

   %let cpopsys = %CPHSTTYP;
   %do i = 1 %to %CPWORDS(&opsysts);
      %if %SCAN(&opsysts,&i)=&cpopsys %then %goto OKSYS;
   %end;
   %put ERROR: CPE2002 IT Service Vision not executable on &cpopsys (not
 one of: &opsysts);
   %let &_rc=12;
   /* stop */
   %goto LOGSCAN;

%OKSYS:
   %let cproot = %cptrimq(&cproot) ;
   %let root = %cptrimq(&root) ;

   %if %QUOTE(&cproot)=%quote() |
       %UPCASE(&resetdic)=YES |
       %UPCASE(&resetopt)=YES
      %then %let cproot = %SCAN(&roots,&i,%STR( ));
   %if %quote(&root)^=%quote() %then %let cproot=%quote(&root);

   %******************************************************************;
   %* Set CPSYSTEM to this opsys if not set by earlier run of CPE     ;
   %******************************************************************;
   %if %STR(&cpsystem)=%STR() %then %let cpsystem = %CPHSTTYP;
   %if %STR(&system)^=%STR()  %then %let cpsystem = %QUPCASE(&system);

   %* Force old GUI for MVS (D0126962);
   %if %STR(&cpsystem)=MVS %then %let guiver=OLD;

   %******************************************************************;
   %* Ensure we know how to process data for the requested system     ;
   %******************************************************************;
   %do i = 1 %to %CPWORDS(&products);
      %if %SCAN(&products,&i)=&cpsystem %then %goto OKPRO;
   %end;
   %put ERROR: CPE2003 &cpsystem data not supported by IT Service Vision
 (not one of: &products);
   %let &_rc=12;
   /* stop */
   %goto LOGSCAN;
%OKPRO:
   %let cpprefix = %SCAN(&prefixes,&i);

   %*********************************************************;
   %* Pass parameters to SCL                                 ;
   %*********************************************************;
   %if %BQUOTE(&mode)=
       %then %let cppmmod=MENU;
       %else %let cppmmod=%BQUOTE(&mode);
   %let profname=%BQUOTE(&remprof);
   %let cppmguiv=%BQUOTE(&guiver);
   %let cppmpdb=%BQUOTE(&pdb);
   %let cppmroo=%BQUOTE(&root);
   %let cppmsys=%BQUOTE(&system);
   %let cpsitesh=%BQUOTE(&siteshr);
   %let cpsiteac=%BQUOTE(&siteacc);
   %let cpconvrt=%BQUOTE(&convert);
   %let cpverrep=%BQUOTE(&verrep);
   %let cppmshr=%BQUOTE(&share);
   %let cppmacc=%BQUOTE(&access);
   %let cppmdsp=%BQUOTE(&disp);
   %if %BQUOTE(&prompt)=
     %then %let cppmpmt=%BQUOTE(NO);
     %else %let cppmpmt=%BQUOTE(&prompt);
   %let cppmint=%BQUOTE(&internal);
   %let cppmsit=%BQUOTE(&sitelib);
   %let cppmrts=%BQUOTE(&rootserv);
   %let cppmmxl=%BQUOTE(&mxglib);
   /*let cppmmxs=%BQUOTE(&mxgsrc);  */
   %let cppmmxs=%BQUOTE(%CMPRES(&mxgsrc));   /*  D0092548 - sxm  */
   %let cppmrc=0;
   %let cpunbuf=%str();

   %********************************************************************;
   %* Do host specific startup to get root name and allocate PGMLIB     ;
   %********************************************************************;
   %&cpprefix.START1( root   = &root,
                      prompt = &prompt,
                      mode   = &mode,
                      _rc    = &_rc );
   %if &&&_rc^=0 %then %goto LOGSCAN ;

   %if &cpopsys=MVS
      %then %let pgmserv=%STR(DISP=SHR) ;
      %else %let pgmserv=%STR(access=readonly) ;

   %if %str(&cp_nmsg)<2 %then
     %do ;
       options %CPSTIMER(no) nonotes ;
     %end ;
   libname pgmlib  "&cproot.pgmlib"  &pgmserv ;
   %if &syslibrc^=0 %then
     %do ;
       %put ERROR: CPE2005 Allocation failed for &cproot.pgmlib.;
       %let &_rc=&syslibrc;
       %goto LOGSCAN;
     %end ;
   options notes %CPSTIMER;

   %********************************************************************;
   %* Allocate CPSYSLIB for new GUI as well for non-MVS products;
   %********************************************************************;
   %if &cpsystem^=MVS %then %do;
      %if &cpopsys=MVS
         %then %let cpsserv=%STR(DISP=SHR) ;
         %else %let cpsserv=%STR(access=readonly) ;

      %if %str(&cp_nmsg)<2 %then
        %do ;
          options %CPSTIMER(no) nonotes ;
        %end ;
      libname cpsyslib  "&cproot.cpsyslib"  &cpsserv ;
      %if &syslibrc^=0 %then
        %do ;
          %put ERROR: CPE2005 Allocation failed for &cproot.cpsyslib.;
          %let &_rc=&syslibrc;
          %goto LOGSCAN;
        %end ;
      options notes %CPSTIMER;
   %end;

   %*********************************************************;
   %* Execute SCL programs to do all other allocations and   ;
   %* verification of data sets in libraries.                ;
   %*********************************************************;
   %let &_rc=16;           %* If not reset by cpstart.scl *;

   %if %UPCASE(&mode)=MENU %then
     %do ;
         %if %INDEX(%STR( &internal ),%STR( PLAYPEN )) %then
           %do ;
             %* Run cpstart.scl from the playpen ;
             dm 'cpe c=playpen.cpemain.cpstart.scl icon=460' continue;
           %end ;
         %else
           %do ;
             %* Run the production cpstart.scl ;
             dm 'cpe c=pgmlib.cpemain.cpstart.scl icon=460' continue;
           %end ;
     %end ;

   %else
     %do ;
       %if %INDEX(%STR( &internal ),%STR( PLAYPEN )) %then
         %do ;
           %* Run cpstart.scl from the playpen ;
           dm 'cpe c=playpen.cpemain.cpstart.scl' ;
         %end ;
       %else
         %do ;
           %* Run the production cpstart.scl ;
           dm 'cpe c=pgmlib.cpemain.cpstart.scl' ;
         %end ;
     %end ;

   %if &&&_rc=0 %then %do;
       %let &_rc=&cppmrc;
       %end;

   %goto LOGSCAN;


   %********************************************************************;
   %*                           THE END                                 ;
   %********************************************************************;

%LOGSCAN:
   ***********************************************************;
   * Close off log and scan for ERROR: at beginning of a line.;
   ***********************************************************;
   %* Flush messages from call executes;
   options %CPSTIMER(no) nonotes nosource ;
   data _null_; run;
   options %CPSTIMER notes source ;

   %* Close off log and scan for ERROR: at beginning of a line.;
   %let scanrc=.;
   %CPLRCHEK ( _rc=scanrc );        %* Scan the log for ERROR: ;
   %if &scanrc>=8 %then %do;
      %put ERROR: Unable to check SAS log for ERROR:.  SCANRC=&scanrc..;
   %end;


%COMP_RC: ;
   ******************************************************************;
   * Return largest value of the various return codes we have at hand;
   ******************************************************************;
   %* Phone home if we caught an unexpected error in the SAS log;
   %if &scanrc^=0 && &&&_rc=0 %then %do;
      %put ERROR: Unexpected error encountered in SAS log.  Please send this
 ALTLOG file to SAS Institute Technical Support.;
      %let &_rc=&scanrc;
   %end;

   %if &&&_rc^=0 %then %do;
      %put NOTE: CPE2006 IT Service Vision ended due to error.;
      %let cppdb=;      %* Ensure next CPSTART will re-init all;
   %end;


%SET_MAX: ;
   *********************************;
   %* Maintain maximum retcode value;
   *********************************;
   %if &CPLRMAX < %EVAL(&&&_rc) %then %let CPLRMAX=%EVAL(&&&_rc);


%EXIT:
   options %CPSTIMER notes source ;

%mend;
